
' General JPEG/JFIF file information reader with EXIF data extraction...

' Tested on 15,000+ image files, 13,500 of which are named as JPEGs. Skips or recovers from bad
' data found in some of these files, and safely handles files erroneously labelled as JPEGs.

' Some references...

' http://www.exif.org/Exif2-2.PDF
' http://www.media.mit.edu/pia/Research/deepview/exif.html
' http://regex.info/exif.cgi

' http://www.takenet.or.jp/~ryuuji/minisoft/exifread/english/ 	-	Exif Reader, for validation -- also crashed on endless recursion file; we don't now!
' http://opanda.com/en/iexif/									-	Another reader for validation

' The first one is useful if you want to add more interpretations in transtag.bmx
' -- search the PDF and transtag.bmx for the tag name you want (eg. XResolution)
' then perform operations on the received 'value' string according to the docs. The
' meanings of EXIF data types are also in the PDF -- important, as Long means Int
' in Blitz, and we can't use unsigned values, though in practice it shouldn't
' matter for files under 2GB in size...

SuperStrict

' Core functions...

Include "jpegreader.bmx"		' PrintJPEGInfo function	(parses 'outer' JPEG file)
Include "exifinfo.bmx"			' PrintEXIFInfo function	(lists EXIF data found)
Include "parseifds.bmx"			' ParseIFD function			(reads EXIF data directories)
Include "tagprocessor.bmx"		' ProcessTag function		(extracts information from tag data)
Include "tagnames.bmx"			' TagName function			(name from tag ID)
Include "transtag.bmx"			' TranslateTag function 	(formatted tag information)

' NB. ParseIFD and ProcessTag recursively call each other! Scary but works fine...

' Support stuff...

Include "helpers.bmx"			' Misc helper stuff
Include "global.bmx"			' Types, consts, globals

' D E M O . . .


' SINGLE FILE DEMO:

' This is not much use if it doesn't contain EXIF data!

	Local img:String

	img = "tay.jpg" ' Demo image shows a few unimplemented tags for fairness...
	
	PrintJPEGInfo img
	
	End

' MULTIPLE FILE DEMO:

' Comment out ^^^ End ^^^ for folder (and sub-folders) demo. Change folder$ below to your image folder...

	Global ImageCount:Long
	
	Local folder$
	
	folder = "C:\My Pictures\"
	
	ParseFolder folder
	
	Print ""
	Print "Tested " + ImageCount + " image files."
	Print ""

	End





' Demo function...

Function ParseFolder (dir:String)

	If Right (dir:String, 1) <> "\" And Right (dir:String, 1) <> "/"
		dir:String = dir:String + "/"
	EndIf
	
	Local folder:Int = ReadDir (dir:String)

	If folder
	
		Repeat

			Local entry:String = NextFile (folder)

			If entry = "" Then Exit
			
			If entry <> "." And entry <> ".."

				Local file:String
				Local full:String
				
				If FileType (dir + entry) = FILETYPE_FILE
	
					file = entry
		
					full = dir
		
					If Right (full, 1) <> "\" And Right (full, 1) <> "/"
						full = full + "\"
					EndIf
		
					full = full + file
		
					ImageCount = ImageCount + 1
					
					'Print ""
					'Print "Reading " + full + " (image number " + ImageCount + ")..."
					
					PrintJPEGInfo full
		
				Else
		
					If FileType (dir + entry) = FILETYPE_DIR
	
						file = entry
		
						If file <> "." And file <> ".."
		
							Local ffolder:String = dir
		
							If Right (ffolder, 1) <> "\" And Right (ffolder, 1) <> "/"
								ffolder = ffolder + "\"
							EndIf
	
							ffolder = ffolder + file
								
							ParseFolder (ffolder)
		
						EndIf
		
					EndIf
		
				EndIf
	
			EndIf

		Forever
	
	EndIf

End Function
