
Function ParseIFD (params:ParameterBundle)

	' ---------------------------------------------------------------
	' Local function for processing tags...
	' ---------------------------------------------------------------
	
	' ---------------------------------------------------------------
	' Image File Directory (IFD) parser (header is in TIFF format)...
	' ---------------------------------------------------------------

	Print ""

	Local dirs:Short
	Local loop:Int
	Local nextifd:Int
	
	Local count:Int = 0
	
	Repeat

		count = count + 1
		
'		Print "DIR COUNT: " + count
		
		' ----------------------------------------------------------
		' Loop through directories, read next IFD offset, abort if 0
		' ----------------------------------------------------------

		TabPrint "--------------------------------------------------------------------------------", params.level
		TabPrint "Reading new IFD", params.level
		TabPrint "--------------------------------------------------------------------------------", params.level
		
'Print ""
'Print "DIRS START at " + StreamPos (params.jpeg)

		dirs = ReadShort (params.jpeg)
		
		If params.endian = ENDIAN_INTEL Then dirs = SwapEndianShort (dirs)

'		Print "Reading " + dirs + " directories..."
		
		Local abort:Int = False
		
		' ----------------------------------------------------------
		' Directory loop...
		' ----------------------------------------------------------

		For loop = 0 Until dirs
		
			' Loop #7 recursing...

			' -----------------------------------------------------
			' 12 bytes per entry...
			' -----------------------------------------------------

			params.tagpos = StreamPos (params.jpeg)
			
			params.tag			= ReadShort (params.jpeg)
			params.dataformat	= ReadShort (params.jpeg)
			params.components	= ReadInt (params.jpeg)
			params.value		= ReadInt (params.jpeg)
			
			If params.endian = ENDIAN_INTEL
				params.tag			= SwapEndianShort (params.tag)
				params.dataformat	= SwapEndianShort (params.dataformat)
				params.components	= SwapEndianInt (params.components)
				params.value		= SwapEndianInt (params.value)
			EndIf

'			Print "TAG AT " + params.tagpos + ": $" + Hex (params.tag)

			' Invalid tag values in J:\My Pictures\Cars\78vauxhall_equus_21.jpg ! Offset 268...
			' Also get this in J:\My Pictures/Music/Pixies/buenos aires 05-02-03.jpg after Interoperability IFD tag...
			
			If params.dataformat = 0 Then Print "INVALID DATA FORMAT ON LOOP " + loop; abort = True; Exit
			
			ProcessTag params

		Next

'		Print "DIRS STOP at " + StreamPos (params.jpeg)

		' ----------------------------------------------------------
		' Assume more IFDs to read if valid data format was found...
		' ----------------------------------------------------------

		If Not abort

			nextifd = ReadInt (params.jpeg) ' THIS IS IT... probably.
			
			If params.endian = ENDIAN_INTEL Then nextifd = SwapEndianInt (nextifd)

			' Seek to next IFD...

			If nextifd
			
				Local seekto:Int = params.tiffstart + nextifd
				
				' Make sure where we're going is within the TIFF section of
				' the JPEG file... or we'll crash/loop forever on broken files...
				
				If seekto < params.tiffstart Or seekto > params.tiffend
					nextifd = 0 ' Invalid TIFF pointer!
				Else
					SeekStream params.jpeg, seekto'; Print "NEXTIFD Seeked to " + StreamPos (params.jpeg)
				EndIf
				
			EndIf
		
		Else
			nextifd = 0 ' Abort, bad data format, assume no more IFDs...
		EndIf
		
	Until nextifd = 0

	TabPrint "--------------------------------------------------------------------------------", params.level
	TabPrint "", params.level
	
End Function


