
Function ProcessTag:String (params:ParameterBundle)

	' ---------------------------------------------------------
	' Reading the tag data...
	' ---------------------------------------------------------

	' "params.value" contains the value we need to interpret; it
	' has already been read and converted to the correct endian-
	' ness. Once interpreted, we stick it in a local string
	' variable, "finalvalue"...
	
	' Unimplemented cases need to be added as you need them.
	' For every data format, datalength is bytespercomponent
	' multiplied by params.components. If datalength is 4 or
	' less, params.value contains the value required and should
	' be interpreted according to the data type. If datalength
	' is greater than 4, store the current StreamPos position,
	' seek ahead to params.tiffstart + dataoffset, then read
	' and interpret the value. Seek back to the previously
	' stored position...

	Local bytespercomponent:Int
	Local datalength:Int
	
	Local formatdesc:String
	Local finalvalue:String = ""
	
	' ---------------------------------------------------------
	' IMPORTANT! Not all fully implemented!
	' ---------------------------------------------------------
	' May be some discrepancies relating to unsigned values
	' and due to interpreting values differently dependent on
	' datalength value (4 or less: use as-is; 5 or more: treat
	' as offset to value)...
	' ---------------------------------------------------------

	' Not all data formats are implemented here, but they appear very
	' uncommon. To implement any of these, see the PDF referenced in
	' demo.bmx for sizes of types, then follow one of the existing
	' examples below...
	
	Select params.dataformat
	
		Case 1
		
			' -----------------------------------------------
			' UNSIGNED BYTE
			' -----------------------------------------------
			
			formatdesc = formatdesc + "unsigned byte (unimplemented)"
			
			bytespercomponent = 1
			datalength = bytespercomponent * params.components

			' This CAN happen! See J:\My Pictures/Backdrops/Windows Backdrops/AU-wp1.jpg
			
			'If datalength > 4 Then Notify "unsigned byte not properly implemented!"; End

			' Print "datalength " + datalength
			Local valueptr:Byte Ptr = Varptr params.value
			
		Case 2

			' -----------------------------------------------
			' ASCII STRING (should be working)
			' -----------------------------------------------

			bytespercomponent = 1
			datalength = bytespercomponent * params.components

			If datalength < 5

				Local valueptr:Byte Ptr = Varptr params.value
				
				For Local loop:Int = 0 Until datalength
					Local char:Byte = valueptr [loop]
					If char
						finalvalue = finalvalue + Chr (char)
					Else
						Exit
					EndIf
				Next
				
			Else
			
				' It's an offset...
				
				Local dataoffset:Int = params.value
				
				' Go read from file offset, then return here...

				Local temp:Int = StreamPos (params.jpeg)
				
				SeekStream params.jpeg, params.tiffstart + dataoffset'; Print "VALUE2 Seeked to " + StreamPos (params.jpeg)

				' Can't just use ReadString -- found example with 0-char before end of datalength...
				
				For Local loop:Int = 0 Until datalength
					Local char:Byte = ReadByte (params.jpeg)
					If char
						finalvalue = finalvalue + Chr (char)
					Else
						Exit
					EndIf
				Next

				SeekStream params.jpeg, temp'; Print "VALUE2ELSE Seeked to " + StreamPos (params.jpeg)
				
			EndIf
															
		Case 3
	
			' -----------------------------------------------
			' UNSIGNED SHORT (should be working)
			' -----------------------------------------------

			bytespercomponent = 2
			datalength = bytespercomponent * params.components

			If datalength < 5

'							finalvalue = Long (Short (params.value))
				finalvalue = Short (params.value)
			
			Else

				Local dataoffset:Int = params.value
				
				Local temp:Int = StreamPos (params.jpeg)

				SeekStream params.jpeg, params.tiffstart + dataoffset'; Print "VALUE3 Seeked to " + StreamPos (params.jpeg)
				
				Local readoff:Short = ReadShort (params.jpeg)
				
				If params.endian = ENDIAN_INTEL
					readoff = SwapEndianShort (readoff)
				EndIf
				
				finalvalue = Int (readoff)

				SeekStream params.jpeg, temp'; Print "VALUE3ELSE Seeked to " + StreamPos (params.jpeg)

			EndIf
			
		Case 4
		
			' -----------------------------------------------
			' UNSIGNED LONG
			' -----------------------------------------------

			bytespercomponent = 4
			datalength = bytespercomponent * params.components
			
			finalvalue = Int (params.value)

		Case 5

			' -----------------------------------------------
			' UNSIGNED RATIONAL (should be working)
			' -----------------------------------------------

			bytespercomponent = 8
			datalength = bytespercomponent * params.components

			If datalength < 5
			
				' Can't happen, two longs = 8 bytes...
				
'							Notify "Oh! It CAN happen... I see. Check unsigned rational conversion with 4 bytes or less!"; End
				
			Else

				' It's an offset...

				Local dataoffset:Int = params.value
				
				Local temp:Int = StreamPos (params.jpeg)

				SeekStream params.jpeg, params.tiffstart + dataoffset'; Print "VALUE5 Seeked to " + StreamPos (params.jpeg)
				
				Local num:Int = ReadInt (params.jpeg)
				Local den:Int = ReadInt (params.jpeg)
				
				If params.endian = ENDIAN_INTEL
					num = SwapEndianInt (num)
					den = SwapEndianInt (den)
				EndIf

				finalvalue = num + "/" + den

				SeekStream params.jpeg, temp'; Print "VALUE5ELSE Seeked to " + StreamPos (params.jpeg)

			EndIf

		Case 6
		
			' -----------------------------------------------
			' SIGNED BYTE
			' -----------------------------------------------

			formatdesc = formatdesc + "signed byte (unimplemented)"
							
			bytespercomponent = 1
			datalength = bytespercomponent * params.components

		Case 7
		
			' -----------------------------------------------
			' UNDEFINED (case-specific)
			' -----------------------------------------------

			bytespercomponent = 1
			datalength = bytespercomponent * params.components

			' Usually implemented as ASCII characters, apparently. Let's wing it and see...
			
			If datalength < 5
			
				Local valueptr:Byte Ptr = Varptr params.value
			
				For Local loop:Int = 0 Until datalength
					finalvalue = finalvalue + Chr (valueptr [loop])
				Next
			
			Else

				' It's an offset...

				Local dataoffset:Int = params.value
				
				Local temp:Int = StreamPos (params.jpeg)
				
				SeekStream params.jpeg, params.tiffstart + dataoffset'; Print "VALUE7ELSE Seeked to " + StreamPos (params.jpeg)
				
					For Local loop:Int = 0 Until datalength
						finalvalue = finalvalue + Chr (ReadByte (params.jpeg))
					Next
				
				SeekStream params.jpeg, temp'; Print "VALUE7ELSE SEEKBACK Seeked to " + StreamPos (params.jpeg)


			EndIf
			
		Case 8
		
			' -----------------------------------------------
			' SIGNED SHORT
			' -----------------------------------------------
			
			formatdesc = formatdesc + "signed short (unimplemented)"
	
			bytespercomponent = 2
			datalength = bytespercomponent * params.components
			
		Case 9
	
			' -----------------------------------------------
			' SIGNED LONG
			' -----------------------------------------------

			bytespercomponent = 4
			datalength = bytespercomponent * params.components

			finalvalue = Int (params.value) ' Signed long = Blitz Int

		Case 10

			' -----------------------------------------------
			' SIGNED RATIONAL (should be working)
			' -----------------------------------------------
			
			bytespercomponent = 8
			datalength = bytespercomponent * params.components

			If datalength < 5
			
				' Can't happen, two longs = 8 bytes...
				
				' Notify "Oh! It CAN happen... I see. Check signed rational conversion with 4 bytes or less!"; End
				
			Else

				' It's an offset...

				Local dataoffset:Int = params.value
				
				Local temp:Int = StreamPos (params.jpeg)

				SeekStream params.jpeg, params.tiffstart + dataoffset'; Print "VALUE10ELSE Seeked to " + StreamPos (params.jpeg)
				
				Local num:Int = ReadInt (params.jpeg)
				Local den:Int = ReadInt (params.jpeg)
				
				If params.endian = ENDIAN_INTEL
					num = SwapEndianInt (num)
					den = SwapEndianInt (den)
				EndIf

				finalvalue = num + "/" + den

				SeekStream params.jpeg, temp'; Print "VALUE10ELSE SEEKBACK Seeked to " + StreamPos (params.jpeg)

			EndIf

		Case 11
		
			' -----------------------------------------------
			' SINGLE FLOAT
			' -----------------------------------------------
			
			formatdesc = formatdesc + "single float (unimplemented)"
	
			bytespercomponent = 4
			datalength = bytespercomponent * params.components

		Case 12
		
			' -----------------------------------------------
			' DOUBLE FLOAT
			' -----------------------------------------------
			
			formatdesc = formatdesc + "double float (unimplemented)"
	
			bytespercomponent = 8
			datalength = bytespercomponent * params.components

		Default
		
			' -----------------------------------------------
			' UNKNOWN DATA FORMAT!
			' -----------------------------------------------

			' Ha, this can happen too! J:\My Pictures/Cars/0_2_FullSize~2.jpg and J:\My Pictures/Cars/78vauxhall_equus_21.jpg
			
			formatdesc = formatdesc + "Unknown data format (invalid)"
			
	End Select

	Local subifd:Int = False ' Used to skip display of information when sub-IFD tag is found...

	Local tagstring:String ' Used in "If Not subifd" section below...

	' ----------------------------------------------------------
	' OK, what kind of tag is it?
	' ----------------------------------------------------------

	Select params.tag

		' -----------------------------------------------------
		' Sub-IFDs: recursively calls ParseIFD on each one...
		' -----------------------------------------------------

		Case $8769, $8825, $A005

			Select params.tag

				Case $8769
					TabPrint "Found Exif SubIFD:", params.level' at offset " + params.tagpos

				Case $8825
					TabPrint "Found GPS Info SubIFD:", params.level' at offset " + params.tagpos

				Case $A005
					TabPrint "Found Interoperability SubIFD:", params.level' at offset " + params.tagpos
					
			End Select

			' This is a subifd, not a standard tag...
			
			subifd = True
			
			' -------------------------------------------
			' Store current position so we can come back...
			' -------------------------------------------

			Local temp:Int = StreamPos (params.jpeg)
'						Print "At pos BEFORE: " + temp
			
			' Hmmm...
			
			Local offset:Int = params.tiffstart + params.value
			Local abort:Int = False
			
			' Check against known-visited "Interoperability SubIFD" file offsets to prevent infinite circular recursion. Ugly...
			
			For Local check:IFDOffset = EachIn IFDsVisited
				If check.offset = offset
					abort = True
					Exit
				EndIf
			Next
			
			If Not abort ' Only do this if not in visited list!
			
				SeekStream params.jpeg, offset'; Print "VALUE PREPARSE Seeked to " + StreamPos (params.jpeg)
				
				' Interoperability SubIFDs may cause circular references, so add to know-visited list...
				
				If params.tag = $A005
					Local off:IFDOffSet = New IFDOffset
					off.offset = offset
					ListAddLast IFDsVisited, off
				EndIf
				
				' ------------------------------------------------
				' Go parse sub-IFD...
				' ------------------------------------------------

				params.level = params.level + 1 ' For TabPrint indenting!
				
					ParseIFD params ' Retro-recursively parse sub-IFD... eek.
					
				params.level = params.level - 1 ' For TabPrint indenting!
				
				' ------------------------------------------------
				' Back to where we left off...
				' ------------------------------------------------
				
				SeekStream params.jpeg, temp'; Print "VALUE POSTPARSE Seeked to " + StreamPos (params.jpeg)
			
'						Else
'							Print "ERROR IN EXIF DATA: Been here before!"
			EndIf
			
'						Print "At pos AFTER: " + temp

		Default
		
			tagstring = "Unimplemented tag [$" + Hex (params.tag) + "] -- add to ProcessTag function!"
		
	End Select

	' ---------------------------------------------------------
	' Valid tag, not a sub-IFD, so print information...
	' ---------------------------------------------------------

	If Not subifd

		tagstring = "~q" + TagName (params.tag) + "~q"

		'TabPrint "Tag $" + Right (Hex (params.tag), 4) + " (data format " + params.dataformat + ")" + " found at offset " + params.tagpos + ":", params.level
		'TabPrint "Tag $" + Right (Hex (params.tag), 4) + ": " + tagstring, params.level
		TabPrint "Tag name:  " + tagstring, params.level

		If finalvalue = ""
			finalvalue = params.value + " (uninterpreted integer, meant to be " + formatdesc + ")"
		EndIf
		
		' Remove these If/EndIf lines to see Makernote junk (can be binary data hence skipped here)...
		
		If params.tag <> $0000927C

'			TabPrint "Value: ~q" + finalvalue + "~q", params.level
			TabPrint "Tag value: ~q" + TranslateTag (params.tag, finalvalue) + "~q", params.level

		EndIf
		
		TabPrint ""
	
	EndIf
	
End Function

