
Function TranslateTag:String (tag:Int, value:String, bonusball:String = "")

	Function Unspec:String (value:String)
		Return "Unspecified or invalid (reserved by EXIF spec): " + value
	End Function

	Local numdom:String [] ' Numerator/denominator in rational number strings...
	
	Select tag
	
		' -----------------------------------------------------
		' IFD0/1 values...
		' -----------------------------------------------------
		
		Case $0100
		
			' "ImageWidth"
			
			Return value
			
		Case $0101
		
			' "ImageLength"
			
			Return value
			
		Case $0102
		
			' "BitsPerSample"
			
			Return value + " bits per sample"
			
		Case $0103
		
			' "Compression"
			
			Select Short (value)
				Case 1
					Return "Uncompressed image data"
				Case 6
					Return "JPEG-compressed thumbnail image data"
				Default
					Return Unspec (value)
			End Select
			
		Case $0106
		
			' "PhotometricInterpretation"
			
			Select Short (value)
				Case 2
					Return "Pixels are in RGB format"
				Case 6
					Return "Pixels are in YCbCr format"
				Default
					Return Unspec (value)
			End Select
			
		Case $010E
		
			' "ImageDescription"
			
			Return value
			
		Case $010F
		
			' "Make"
			
			Return value
			
		Case $0110
		
			' "Model"
			
			Return value
			
		Case $0111
			' "StripOffsets"
			Return value
			
		Case $0112
		
			' "Orientation"
			
			' NOTE: These may be incorrect! Struggling to work them out from spec descriptions!
			
			' The rotated values might be flipped too... need to test images with these values...
			
			Select value
				Case 1
					Return "Normal"
				Case 2
					Return "Horizontally flipped"
				Case 3
					Return "Horizontally and vertically flipped"
				Case 4
					Return "Vertically flipped"
				Case 5
					Return "Rotated... somehow..."
				Case 6
					Return "Rotated... somehow..."
				Case 7
					Return "Rotated... somehow..."
				Case 8
					Return "Rotated... somehow..."
				Default
					Return Unspec (value)
			End Select
			
		Case $0115
			' "SamplesPerPixel"
			Return value + " samples per pixel"
			
		Case $0116
			' "RowsPerStrip"
			Return value
			
		Case $0117
			' "StripByteCounts"
			Return value
		
		' BELOW: Re. XResolution and YResolution. Since we don't know if ResolutionUnit
		' has been read yet, if you need this data to be correctly returned, scan the
		' whole file for the EXIF data you're interested in, store it, THEN interpret it.
		
		' So, for this information, you would store XResolution, YResolution, and
		' ResolutionUnit. Once the file has been fully read, you can then call
		' TranslateTag with the string value of XResolution in the 'value' parameter
		' and the string value of ResolutionUnit in the 'bonusball' parameter to get
		' the correct results.
		
		' (Careful if you try to perform division on 'value' (eg. "72/1") as I've found
		' at least one photo with invalid data that results in a divide-by-zero error!)
		
		Case $011A

			' "XResolution"

			Select bonusball
				Case "2"
					Return value + " pixels per inch"
				Case "3"
					Return value + " pixels per cm"
				Default
					Return value + " pixels per inch (assumed, as per spec)"
			End Select
			
		Case $011B

			Select bonusball
				Case ""
					Return value + " pixels per inch (assumed, as per spec)"
				Case "2"
					Return value + " pixels per inch"
				Case "3"
					Return value + " pixels per cm"
				Default
					Return Unspec (value)
			End Select
			
		Case $011C
			
			' "PlanarConfiguration"
			
			Select value
				Case 1
					Return "Pixels are in 'chunky' format"
				Case 2
					Return "Pixels are in 'planar' format"
				Case 3
					Return Unspec (value)
			End Select
			
		Case $0128
		
			' "ResolutionUnit"
		
			Select value
				Case ""
					Return "inches (assumed, as per spec)"
				Case 2
					Return "inches"
				Case 3
					Return "cm"
				Default
					Return Unspec (value)
			End Select

		Case $012D
			' "TransferFunction"
			Return value

		Case $0131
		
			' "Software"
		
			Return value

		Case $0132
		
			' "DateTime"
		
			Return value

		Case $013B
		
			' "Artist"
		
			Return value

		Case $013E
			' "WhitePoint"
			Return value

		Case $013F
			' "PrimaryChromaticities"
			Return value

		Case $0201
		
			' "JPEGInterchangeFormat"
		
			Return "JPEG-compressed thumbnail data stored at byte offset #" + value

		Case $0202
		
			' "JPEGInterchangeFormatLength"

			If value = 0
				Return "Empty JPEG-compressed thumbnail data"
			Else
				Return "JPEG-compressed thumbnail data stored in " + value + " bytes"
			EndIf

		Case $0211
			' "YCbCrCoefficients"
			Return value

		Case $0212
			' "YCbCrSubSampling"
			Return value

		Case $0213
			
			' "YCbCrPositioning"
			
			' Probably means something to somebody...
			
			Select value
				Case 1
					Return "Chrominance components centered in relation to luminance"
				Case 1
					Return "Chrominance components co-sited with luminance"
			End Select

		Case $0214
			' "ReferenceBlackWhite"
			Return value

		Case $8298
			' "Copyright"
			Return value

		' -----------------------------------------------------
		' EXIF Sub-IFD values...
		' -----------------------------------------------------
		
		Case $0000829A
			' "ExposureTime"
			Return value

		Case $0000829D
			' "FNumber"
			Return value

		Case $00008822
		
			' "ExposureProgram"
		
			Select value
				Case 0
					Return "Not defined"
				Case 1
					Return "Manual exposure"
				Case 2
					Return "Normal"
				Case 3
					Return "Aperture priority"
				Case 4
					Return "Shutter priority"
				Case 5
					Return "Creative"
				Case 6
					Return "Action"
				Case 7
					Return "Portraits"
				Case 8
					Return "Landscapes"
				Default
					Return Unspec (value)
			End Select

		Case $00008824
			' "SpectralSensitivity"
			Return value

		Case $00008827
			' "ISOSpeedRatings"
			Return value

		Case $00008828
			' "OECF"
			Return value

		Case $00009000
		
			' "ExifVersion"
			
			Local version:String = Int (Left (value, 2)) + "." + Int (Right (value, 2))
			
			' Not widely tested!
			
			While Right (version, 1) = "0"
				version = Left (version, Len (version) - 1)
				If version = "" Then version = "Unknown"; Exit
			Wend
			
			' 0220 becomes 2.2, 0221 becomes 2.21, etc...
			
			Return version
				
		Case $00009003
			' "DateTimeOriginal"
			Return value

		Case $00009004
			' "DateTimeDigitized"
			Return value

		Case $00009101
			' "ComponentsConfiguration"
			Return value

		Case $00009102
			' "CompressedBitsPerPixel"
			Return value

		Case $00009201
		
			' "ShutterSpeedValue"
		
			Return value

		Case $00009202

			' "ApertureValue"
			
			Return value

		Case $00009203
			' "BrightnessValue"
			Return value

		Case $00009204
			' "ExposureBiasValue"
			Return value

		Case $00009205
			' "MaxApertureValue"
			Return value

		Case $00009206
			' "SubjectDistance"
			Return value

		Case $00009207
			' "MeteringMode"
			Return value

		Case $00009208
			' "LightSource"
			Return value

		Case $00009209
			' "Flash"
			Return value

		Case $0000920A
			' "FocalLength"
			Return value

		Case $00009214
			' "SubjectArea"
			Return value

		Case $0000927C
			' "MakerNote"
			Return value

		Case $00009286
			' "UserComment"
			Return value

		Case $00009290
			' "SubSecTime"
			Return value

		Case $00009291
			' "SubSecTimeOriginal"
			Return value

		Case $00009292
			' "SubSecTimeDigitized"
			Return value

		Case $0000A000
			' "FlashPixVersion"
			Return value

		Case $0000A001
		
			' "ColorSpace"
			
			Select value
				Case 1
					Return "sRGB color space"
				Case $FFFF
					Return "Uncalibrated color space"
				Default
					Return Unspec (value)
			End Select

		Case $0000A002
			' "ExifImageWidth"
			Return value

		Case $0000A003
			' "ExifImageHeight"
			Return value

		Case $0000A004
			' "RelatedSoundFile"
			Return value

		Case $0000A20B
			' "FlashEnergy"
			Return value

		Case $0000A20C
			' "SpatialFrequencyResponse"
			Return value

		Case $0000A20E
			' "FocalPlaneXResolution"
			Return value

		Case $0000A20F
			' "FocalPlaneYResolution"
			Return value

		Case $0000A210
			' "FocalPlaneResolutionUnit"
			Return value

		Case $0000A214
			' "SubjectLocation"
			Return value

		Case $0000A215
			' "ExposureIndex"
			Return value

		Case $0000A217
			' "SensingMethod"
			Return value

		Case $0000A300
			' "FileSource"
			Return value

		Case $0000A301
			' "SceneType"
			Return value

		Case $0000A302
			' "CFAPattern"
			Return value

		Case $0000A401
			' "CustomRendered"
			Return value

		Case $0000A402
			' "ExposureMode"
			Return value

		Case $0000A403
			' "WhiteBalance"
			Return value

		Case $0000A404
			' "DigitalZoomRatio"
			Return value

		Case $0000A405
			' "FocalLengthIn35mmFilm"
			Return value

		Case $0000A406
			' "SceneCaptureType"
			Return value

		Case $0000A407
			' "GainControl"
			Return value

		Case $0000A408
			' "Contrast"
			Return value

		Case $0000A409
			' "Saturation"
			Return value

		Case $0000A40A
			' "Sharpness"
			Return value

		Case $0000A40B
			' "DeviceSettingDescription"
			Return value

		Case $0000A40C
			' "SubjectDistanceRange"
			Return value

		Case $0000A420
			' "ImageUniqueID"
			Return value

		' -----------------------------------------------------
		' GPS sub-IFD values...
		' -----------------------------------------------------
	
		Case $0
			' "GPSVersionID"
			Return value

		Case $1
			' "GPSLatitudeRef"
			Return value

		Case $2
			' "GPSLatitude"
			Return value

		Case $3
			' "GPSLongitudeRef"
			Return value

		Case $4
			' "GPSLongitude"
			Return value

		Case $5
			' "GPSAltitudeRef"
			Return value

		Case $6
			' "GPSAltitude"
			Return value

		Case $7
			' "GPSTimeStamp"
			Return value

		Case $8
			' "GPSSatellites"
			Return value

		Case $9
			' "GPSStatus"
			Return value

		Case $A
			' "GPSMeasureMode"
			Return value

		Case $B
			' "GPSDOP"
			Return value

		Case $C
			' "GPSSpeedRef"
			Return value

		Case $D
			' "GPSSpeed"
			Return value

		Case $E
			' "GPSTrackRef"
			Return value

		Case $F
			' "GPSTrack"
			Return value

		Case $10
			' "GPSImgDirectionRef"
			Return value

		Case $11
			' "GPSImgDirection"
			Return value

		Case $12
			' "GPSMapDatum"
			Return value

		Case $13
			' "GPSDestLatitudeRef"
			Return value

		Case $14
			' "GPSDestLatitude"
			Return value

		Case $15
			' "GPSDestLongitudeRef"
			Return value

		Case $16
			' "GPSDestLongitude"
			Return value

		Case $17
			' "GPSDestBearingRef"
			Return value

		Case $18
			' "GPSDestBearing"
			Return value

		Case $19
			' "GPSDestDistanceRef"
			Return value

		Case $1A
			' "GPSDestDistance"
			Return value

		Case $1B
			' "GPSProcessingMethod"
			Return value

		Case $1C
			' "GPSAreaInformation"
			Return value

		Case $1D
			' "GPSDateStamp"
			Return value

		Case $1E
			' "GPSDifferential"
			Return value

	End Select

End Function

